 /*---------------------------------------------------------------------------*\
 
    FILE....: hellodigi.c
    AUTHOR..: Paulo Garcia
    DATE....: 27/02/2004
 
    This file contains the diagnostic tool for Digivoice Cards
 
    This program requires ncurses

---------------------------------------------------------------------------*/
 
/*---------------------------------------------------------------------------*\

         hellodigi - LGPL Automatic Call Interceptor

         Copyright (C) 2001 Digivoice www.digivoice.com.br

         This library is free software; you can redistribute it and/or
         modify it under the terms of the GNU Lesser General Public
         License as published by the Free Software Foundation; either
         version 2.1 of the License, or (at your option) any later version.

         This library is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
         Lesser General Public License for more details.

         You should have received a copy of the GNU Lesser General Public
         License along with this library; if not, write to the Free Software
         Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	 USA.

\*---------------------------------------------------------------------------*/

#pragma pack(1)


#define 	MAX_CHAN	360

//constants used in call_state

#define 	IDLE		0
#define		ANSWERED	1
#define		PLAYING		2




#include "../../voicerlib.h"
#include "../../vppci/vppci.h"

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <pthread.h>
#include <getopt.h>
#include <ncurses.h>

int count=0;
int current_channel=1;

int file_format = 0;	//ffWave,ffSig,ffWavePCM, ffGsm610

char *szFF[] = { "ffWaveULaw\0","ffSig\0","ffWavePCM\0", "ffGsm610\0", "ffWaveALaw\0" };

char szFileName[255] = { "./welcome.wav" };
int card_type;

short call_state[MAX_CHAN];



extern pthread_mutex_t event_mutex;



dg_event_data_strucuture event_context;

void print_event(char *szMsg, int port)
{
	
	struct tm *now;
	time_t curSecs;
	curSecs = time(NULL);				
	now = localtime(&curSecs);
	
	mvprintw(4+port,1,"<%02d:%02d:%02d> port %d: %s\n",now->tm_hour, now->tm_min, now->tm_sec, port,szMsg);
	
}
//---------- end of NCURSES screen functions -----------

//----------------------------------------------------------------------------------------
// Functions to handle alarm information - Will be embedded into voicerlib soon
//----------------------------------------------------------------------------------------


//------------------------------------------------------------------------------------------
// Function that will be receive events from library 
//------------------------------------------------------------------------------------------
void ReceiveEvents(void *context_data)
{
    dg_event_data_strucuture *EventContext;
    char data[255];
    char szDig[MAX_PATH];
    short 	ret;
    data[0] = 0;
    
        
    /* Copy received Data */
    //pthread_mutex_lock(&event_mutex);
    EventContext = ((dg_event_data_strucuture*)context_data);
    //pthread_mutex_unlock(&event_mutex);
 
		switch (EventContext->command)
		{
			case C_R2:
			    sprintf(data,"R2 Received - %x",EventContext->data);
			    break;
			case C_ALARME:
				//The variable port contains the Card (hi nibble) and Line (e1-a ou e1-b) (low nibble)
			    sprintf(data,"Alarm Received Card/E1: %d - Code: %x",EventContext->port,
									     								EventContext->data);
																		
				//The alarm information is passed in a bit-based byte
				//-------------------------------------------------------------------
				/*  
					ALARM_RSLIP		= 0x00000001,		//escorregamento 
					ALARM_RAIS		= 0x00000002,		//alarme remoto
					ALARM_AISS		= 0x00000004,		//indicao de alarme
					ALARM_AIS16S	= 0x00000008,		//indicao de alarme canal  16
					ALARM_LOSS		= 0x00000010,		//perda de sinal
					ALARM_RESERVED  = 0x00000020,		//reservado
					ALARM_MFSYNC	= 0x00000040,		//sincronismo de multiquadro
					ALARM_SYNC		= 0x00000080,		//sincronismo de quadro
				*/
				//-------------------------------------------------------------------
				
			    break;
			case EV_E1CHANGESTATUS:
				//this events are related only for E1 R2 protocol
				switch (EventContext->data)
				{
					case C_SEIZURE:
						//R2 protocol. wait for 4 digits
						dg_sete1rxcount(EventContext->port, 4);
						break;
					case C_NUMERO_RECEBIDO:
						dg_gete1number(EventContext->port, szDig);
						sprintf(data,"E1 Events: Number received = %s", szDig);
						break;
					case C_CONGESTION:
						sprintf(data,"E1 Events: Congestion");
						break;
					case C_B_ENDCALL:
					  sprintf(data,"E1 Events: B Subscriber hangup");
						break;
					case C_NOTCOMPLETED:
						sprintf(data,"E1 Events: Call couldt be completed");
						break;
					case C_CATEGORIA:
						sprintf(data,"E1 Events: Category received");
						break;
					case C_UNAVAILABLE:
					  sprintf(data,"E1 Events: Channel unavailable");
						break;
				}
				break;
			case EV_CALLERID:
				dg_getcallerid(EventContext->port,szDig);
				sprintf(data,"CallerID: %s", szDig);
				break;
			case EV_RING:
				sprintf(data,"RING", EventContext->port);

				//answer the call with 100ms AFTER ANSWER PAUSE 
				dg_pickup(EventContext->port,100);
				call_state[EventContext->port] = ANSWERED;

				break;       
			case EV_BUSY:
				sprintf(data,"BUSY");
				dg_stopplayfile(EventContext->port);
				dg_hangup(EventContext->port);
				call_state[EventContext->port] = IDLE;

				break;
			case EV_AFTERPICKUP:
				sprintf(data,"Playing %s...",szFileName);
				ret = dg_playfile(EventContext->port,szFileName,"",0);
				if (ret==0)
				{
					call_state[EventContext->port] = ANSWERED;
				}
				else
				{	
					//error
					dg_hangup(EventContext->port);
					call_state[EventContext->port] = IDLE;
				}
			break;
			case EV_PLAYSTOP:
				//end call
				sprintf(data,"Play Stop EVENT");
				dg_hangup(EventContext->port);
				call_state[EventContext->port] = IDLE;
				dg_send_r2(EventContext->port,0x9);

			break;
		}
//	if (data[0]!=0)
		print_event(data, EventContext->port);
}

//----------------------------------------------------------
//Shows help messages
//----------------------------------------------------------
void show_help()
{
		printf("\nInvalid Parameters!");
		printf("\nUsage: hellodigi -c <num_cards> -t <card_type> -p <firmware_path>");
		printf("\n       Where: num_cards - Number of installed cards     ");
		printf("\n              card_type - vbox or vppci     ");
		printf("\n              firm_path - Path to the firmware path\n");
}

//------------------------------------------------------------------------------------------
// Main Function 
//------------------------------------------------------------------------------------------
int main (int argc, char *argv[])
{
	char ch;
	char szNumber[2];
	char szTemp[255];
	char szMsg[255];
	const char* program_name;
	int next_option,ret;
	int card_count;
	short paused=0;
	short i;
  		
	if (argc < 4)
	{
		show_help();
		exit(EXIT_FAILURE);
	}
	
	
	
	/* Parse Command line arguments */
	/* parameters: no_cards , card_type, szFirmwarePath */ 
	const char* const short_options = "c:t:p:";
	
	const struct option long_options[] = {
		{ "type", 1, NULL, 't'}, 		
		{ "path", 1, NULL, 'p'}, 		
		{ NULL, 0, NULL, 0 }
		};
		
	const char output_filename = (const char)NULL;

	//-------------------------------------------------------------------
	// command line parser
	//-------------------------------------------------------------------
	program_name = argv[0]; /* Saves program name*/
		
	do {
		next_option = getopt_long(argc, argv, short_options, long_options, NULL);
		
		switch(next_option)
		{
		
			case 't':
				sscanf(optarg,"%s",&szTemp);
				//check card type
				if (!strncmp(szTemp,"vbox",4))
					card_type = (short)ctVBPCI;
				else
					if (!strncmp(szTemp,"vppci",5))
						card_type = (short)ctVPPCI;
					else
							if (!strncmp(szTemp,"vbe1",4))
								card_type = (short)ctVBE1;
							else						
							{
								show_help();
								exit(EXIT_FAILURE);
							}
				printf("\nTipo de Placa: %d\n",card_type);
				break;
			case 'p':		/* -p or --path */
				
				sscanf(optarg,"%s",&szTemp);				
				if (strlen(szTemp)==0)
				{
						show_help();
						exit(EXIT_FAILURE);
				}
				break;
		}
	}
	while (next_option != -1);
	//-------------------------------------------------------------------
		
	//path to firmware	
	printf("Locating firmware at %s...\n",szTemp);
	
			
	/* Start VoicerLib engine */
	card_count = 1;
	if ((ret = dg_startvoicerlib(card_count,card_type,szTemp)) != EXIT_SUCCESS)
	{
			printf("\nError starting voicerlib (code %d)\n",ret);
			exit(EXIT_FAILURE);
	}
	card_count = dg_getcardscount();
	printf("Card Count %d...\n",card_count);
	if (card_type != ctVBE1)
	{
	/* Sets frequency time parameter. default 1500ms*/
		dg_setfrequencytime(1500);
	}
	/* Set events function callback */
	dg_seteventfunction(ReceiveEvents,&event_context);
	sleep(2);
	
	//CONFIGURE CARDS BEHAVIOUR				
	//card 0 is STBUS master and sync in E1-A
	printf("Setting H100 defaults. External SYNC....\n",szTemp);
	for (i=1;i<card_count;i++)
	{
		dg_seth100sync(1,H100_CT_NETREF,H100_REF_SOURCE1,H100_REF_L_NETREF0);
		dg_seth100sync(1,H100_CT_NETREF,H100_REF_DIV1,H100_REF_D256);
		dg_seth100sync(1,H100_MASTER, H100_MS_REFFR,H100_MS_8KHz);
		dg_seth100sync(1,H100_MASTER, H100_MS_REF,H100_MS_CT_NETREF1);
		dg_seth100sync(1,H100_MASTER, H100_MS_SEL,H100_MS_CT_A);
		dg_seth100sync(1,H100_SLAVE, H100_SL_CT_A,0);
	}	
	printf("Setting General Defaults....\n");
	//default properties for each enabled port
	//Enabling E1 Handle, allows the program to answer E1 public trunks
	for(current_channel=1; current_channel<=dg_getportscount();current_channel++)
	{
		dg_create_e1_handle(current_channel);
		//set the same id for all channels
		strcpy(szTemp,"1141952557\0");
		dg_setportid(current_channel,szTemp);
		//set the same category  for all channels
		dg_setportcategory(current_channel,1);

		call_state[current_channel] = IDLE;

		dg_setplayformat(current_channel,ffWaveULaw);
	}
	current_channel=1;
	
	
	/* initialize ncurses stuff */
	initscr();
	cbreak();       /* Line buffering disabled, Pass onevery thing to me         */
	keypad(stdscr, TRUE);        /* I need that nifty F1     */
	noecho();
	

	/* Main Loop */
	mvprintw(0,30,"   * Digivoice VoicerLib Diagnostic Tool *  ");
	mvprintw(2,1,"Waiting for calls....(Press <q> to quit)\n");	
	mvprintw(3,1,"Using %s greeting message.\n",szFileName);

	while((ch = getch()) != 'q')
	{    
		
		if (ch == 'Q')
		    break;
	  
		count++;  //Remove in the future
    } /* while */     

	for(current_channel=1; current_channel<=dg_getportscount();current_channel++)
	{
		dg_destroy_e1_handle(current_channel);
	}
    /* Stops voicerlib engine */
    dg_shutdownvoicerlib();
	endwin();
    printf("\n\n* * * Goodbye! * * *\n\n");
    return EXIT_SUCCESS;
}

